import os
from typing import Optional

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.database import DatabaseSettings
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironment
from cpl_discord import get_discord_collection
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from dotenv import load_dotenv

from bot.startup_settings_extension import StartupSettingsExtension
from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.custom_file_logger_abc import CustomFileLoggerABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.bot_logging_settings import BotLoggingSettings
from bot_core.logging.command_logger import CommandLogger
from bot_core.logging.database_logger import DatabaseLogger
from bot_core.logging.message_logger import MessageLogger
from bot_core.pipes.date_time_offset_pipe import DateTimeOffsetPipe
from bot_core.service.client_utils_service import ClientUtilsService
from bot_core.service.message_service import MessageService
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.statistic_repository_abc import StatisticRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.db_context import DBContext
from bot_data.service.auth_user_repository_service import AuthUserRepositoryService
from bot_data.service.auto_role_repository_service import AutoRoleRepositoryService
from bot_data.service.client_repository_service import ClientRepositoryService
from bot_data.service.known_user_repository_service import KnownUserRepositoryService
from bot_data.service.level_repository_service import LevelRepositoryService
from bot_data.service.seeder_service import SeederService
from bot_data.service.server_repository_service import ServerRepositoryService
from bot_data.service.statistic_repository_service import StatisticRepositoryService
from bot_data.service.user_joined_server_repository_service import UserJoinedServerRepositoryService
from bot_data.service.user_joined_voice_channel_service import UserJoinedVoiceChannelRepositoryService
from bot_data.service.user_repository_service import UserRepositoryService
from ui_tests.test_on_ready_event import TestOnReadyEvent


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)

        self._config: Optional[ConfigurationABC] = None

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        load_dotenv()
        configuration.add_environment_variables('KDB_TEST_')
        configuration.add_environment_variables('DISCORD_')

        cwd = os.path.dirname(os.path.realpath(__file__))
        configuration.add_json_file(f'{cwd}/config/appsettings.json', optional=False)
        configuration.add_json_file(f'{cwd}/config/appsettings.{environment.host_name}.json', optional=True)

        StartupSettingsExtension._configure_settings_with_sub_settings(configuration, BotLoggingSettings, lambda x: x.files, lambda x: x.key)

        self._config = configuration
        return configuration

    def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_logging()
        services.add_singleton(CustomFileLoggerABC, CommandLogger)
        services.add_singleton(CustomFileLoggerABC, DatabaseLogger)
        services.add_singleton(CustomFileLoggerABC, MessageLogger)

        services.add_translation()
        db_settings: DatabaseSettings = self._config.get_configuration(DatabaseSettings)
        db_settings_with_pw = DatabaseSettings()
        pw = self._config.get_configuration('DB_PASSWORD')
        db_settings_with_pw.from_dict({
            "Host": db_settings.host,
            "User": db_settings.user,
            "Password": '' if pw is None else pw,
            "Database": db_settings.database,
            "Charset": db_settings.charset,
            "UseUnicode": db_settings.use_unicode,
            "Buffered": db_settings.buffered,
            "AuthPlugin": db_settings.auth_plugin
        })
        services.add_db_context(DBContext, db_settings_with_pw)

        services.add_discord()
        dc = get_discord_collection(services)
        dc.add_event(DiscordEventTypesEnum.on_ready.value, TestOnReadyEvent)

        # bot_core stuff
        services.add_transient(MessageServiceABC, MessageService)
        services.add_transient(ClientUtilsServiceABC, ClientUtilsService)

        # pipes
        services.add_transient(DateTimeOffsetPipe)

        # data stuff
        services.add_transient(AuthUserRepositoryABC, AuthUserRepositoryService)
        services.add_transient(ServerRepositoryABC, ServerRepositoryService)
        services.add_transient(UserRepositoryABC, UserRepositoryService)
        services.add_transient(ClientRepositoryABC, ClientRepositoryService)
        services.add_transient(KnownUserRepositoryABC, KnownUserRepositoryService)
        services.add_transient(UserJoinedServerRepositoryABC, UserJoinedServerRepositoryService)
        services.add_transient(UserJoinedVoiceChannelRepositoryABC, UserJoinedVoiceChannelRepositoryService)
        services.add_transient(AutoRoleRepositoryABC, AutoRoleRepositoryService)
        services.add_transient(LevelRepositoryABC, LevelRepositoryService)
        services.add_transient(StatisticRepositoryABC, StatisticRepositoryService)

        services.add_transient(SeederService)

        return services.build_service_provider()
